import React, { useState } from "react";
import { keyEnter, keyTab } from "@microsoft/fast-web-utilities";
import { getDataFromSchema } from "design-to-code";
import { DragItem } from "../../templates";
import { LinkedDataActionType } from "../../templates/types";
import cssVariables from "design-to-code/dist/stylesheets/web-components/style/global.css-variables.css";
import cleanListStyle from "design-to-code/dist/stylesheets/web-components/style/common.clean-list.css";
import ellipsisStyle from "design-to-code/dist/stylesheets/web-components/style/common.ellipsis.css";
import removeStyle from "design-to-code/dist/stylesheets/web-components/style/common.remove.css";
import removeItemStyle from "design-to-code/dist/stylesheets/web-components/style/common.remove-item.css";
import style from "./control.linked-data.style.css";
import dtcClassName from "design-to-code/dist/esm/web-components/style/class-names";
// tree-shaking
cssVariables;
cleanListStyle;
ellipsisStyle;
removeStyle;
removeItemStyle;
style;
/**
 * Form control definition
 */
function LinkedDataControl(props) {
    const [searchTerm, setSearchTerm] = useState("");
    const [dragging, setDragging] = useState(false);
    const [data, setData] = useState([].concat(props.value || []));
    /**
     * Render the UI for adding linked data
     */
    function renderAddLinkedData() {
        return (React.createElement("div", { className: "dtc-linked-data-control_linked-data-list-control" },
            React.createElement("span", { className: `dtc-linked-data-control_linked-data-list-input-region ${dtcClassName.commonSelectSpan}` },
                React.createElement("input", { className: `dtc-linked-data-control_linked-data-list-input ${dtcClassName.commonSelectInput}`, type: "text", "aria-autocomplete": "list", list: getLinkedDataInputId(), "aria-controls": getLinkedDataInputId(), value: searchTerm, placeholder: props.strings.linkedDataPlaceholder, onChange: handleSearchTermUpdate, onKeyDown: handleLinkedDataKeydown })),
            React.createElement("datalist", { id: getLinkedDataInputId(), role: "listbox" }, renderFilteredLinkedDataOptions())));
    }
    function renderFilteredLinkedDataOptions() {
        return Object.entries(props.schemaDictionary).map(([key, value]) => {
            return (React.createElement("option", { key: key, value: value.title, label: typeof value.alias === "string" ? value.alias : void 0 }));
        });
    }
    /**
     * Render the list of existing linkedData for a component
     */
    function renderExistingLinkedData() {
        const childItems = renderExistingLinkedDataItem();
        if (childItems) {
            return (React.createElement("ul", { className: `dtc-linked-data-control_existing-linked-data ${dtcClassName.commonCleanList}` }, childItems));
        }
    }
    function renderExistingLinkedDataItem() {
        if (Array.isArray(props.value)) {
            return (dragging ? data : props.value).map((value, index) => {
                return (React.createElement(DragItem, { key: value + index, itemClassName: "dtc-linked-data-control_existing-linked-data-item", itemLinkClassName: `dtc-linked-data-control_existing-linked-data-item-link ${dtcClassName.commonEllipsis}`, itemRemoveClassName: `dtc-linked-data-control_delete-button ${dtcClassName.commonRemoveItem}`, minItems: 0, itemLength: 1, index: index, onClick: handleItemClick(value.id), removeDragItem: handleRemoveItem, moveDragItem: handleMoveItem, dropDragItem: handleDropItem, dragStart: handleDragStart, dragEnd: handleDragEnd, strings: props.strings }, props.schemaDictionary[props.dataDictionary[0][value.id].schemaId].title));
            });
        }
    }
    function handleDragStart(config) {
        setDragging(true);
        setData([].concat(props.value || []));
        return config;
    }
    function handleDragEnd() {
        setDragging(false);
    }
    function handleItemClick(id) {
        return (index) => {
            return (e) => {
                props.onUpdateSection(id);
            };
        };
    }
    function handleRemoveItem(type, index) {
        return (e) => {
            props.onChange({
                value: props.value.splice(index, 1),
                isLinkedData: true,
                linkedDataAction: LinkedDataActionType.remove,
            });
        };
    }
    function handleMoveItem(sourceIndex, targetIndex) {
        const currentData = [].concat(props.value);
        if (sourceIndex !== targetIndex) {
            currentData.splice(targetIndex, 0, currentData.splice(sourceIndex, 1)[0]);
        }
        setData(currentData);
    }
    function handleDropItem() {
        props.onChange({
            value: data,
            isLinkedData: true,
            linkedDataAction: LinkedDataActionType.reorder,
        });
    }
    function handleLinkedDataKeydown(e) {
        if (e.target === e.currentTarget) {
            // Enter adds linked data if the input value matches a schema lazily or exactly
            if (e.key === keyEnter) {
                e.preventDefault();
                const normalizedValue = e.currentTarget.value.toLowerCase();
                if (lazyMatchValueWithASingleSchema(normalizedValue) ||
                    matchExactValueWithASingleSchema(e.currentTarget.value)) {
                    addLinkedData(normalizedValue, e.currentTarget.value);
                    /**
                     * Adding items to the linked data causes the items to
                     * move the input down while the datalist remains in the same location,
                     * to prevent the datalist from overlapping the input
                     * the datalist is dismissed by defocusing and refocusing the input
                     */
                    e.target.blur();
                    e.target.focus();
                    setSearchTerm("");
                }
                // Tab performs an auto-complete if there is a single schema it can match to
            }
            else if (e.key === keyTab) {
                const normalizedValue = e.currentTarget.value.toLowerCase();
                const matchedSchema = lazyMatchValueWithASingleSchema(normalizedValue);
                if (typeof matchedSchema === "string") {
                    // prevent navigating away by tab when single schema matched
                    e.preventDefault();
                    setSearchTerm(props.schemaDictionary[matchedSchema].title);
                }
            }
        }
    }
    function lazyMatchValueWithASingleSchema(value) {
        const matchingSchemas = Object.keys(props.schemaDictionary).reduce((previousValue, currentValue) => {
            if (props.schemaDictionary[currentValue].title.toLowerCase().includes(value)) {
                return previousValue.concat([currentValue]);
            }
            return previousValue;
        }, []);
        if (matchingSchemas.length === 1) {
            return matchingSchemas[0];
        }
    }
    function matchExactValueWithASingleSchema(value) {
        return Object.keys(props.schemaDictionary).find((schemaDictionaryKey) => {
            return value === props.schemaDictionary[schemaDictionaryKey].title;
        });
    }
    /**
     * Change handler for editing the search term filter
     */
    function handleSearchTermUpdate(e) {
        setSearchTerm(e.target.value);
    }
    function addLinkedData(normalizedValue, originalValue) {
        const matchedNormalizedValue = lazyMatchValueWithASingleSchema(normalizedValue);
        const matchedOriginalValue = matchExactValueWithASingleSchema(originalValue);
        const schemaId = matchedNormalizedValue || matchedOriginalValue;
        if (typeof schemaId !== "undefined") {
            props.onChange({
                value: [
                    {
                        schemaId,
                        parent: {
                            id: props.dictionaryId,
                            dataLocation: props.dataLocation,
                        },
                        data: getDataFromSchema(props.schemaDictionary[schemaId], props.schemaDictionary[schemaId]),
                    },
                ],
                isLinkedData: true,
                linkedDataAction: LinkedDataActionType.add,
                index: Array.isArray(props.value) ? props.value.length : 0,
            });
        }
    }
    function getLinkedDataInputId() {
        return `${props.dataLocation}-input`;
    }
    // Convert to search component when #3006 has been completed
    return (React.createElement("div", { className: "dtc-linked-data-control" },
        renderExistingLinkedData(),
        renderAddLinkedData()));
}
export { LinkedDataControl };
export default LinkedDataControl;
//# sourceMappingURL=control.linked-data.js.map