import React from "react";
import { isEqual } from "lodash-es";
import Badge from "./badge";
import DefaultValue from "./default-value";
import ConstValue from "./const-value";
import Remove from "./remove";
import { ControlType } from "./types";
export function renderRemove(props) {
    if (!props.required &&
        props.remove &&
        props.data !== undefined &&
        props.type !== ControlType.linkedData) {
        return (React.createElement(Remove, { className: props.className, onChange: handleRemove(props), disabled: props.disabled && props.data === undefined }));
    }
}
export function renderBadge(props) {
    if (props.badge) {
        return (React.createElement(Badge, { className: props.className, type: props.badge, description: props.badgeDescription }));
    }
}
export function renderConstValueIndicator(props) {
    if (props.const !== undefined && props.data !== props.const) {
        return (React.createElement(ConstValue, { className: props.className, disabled: props.disabled, onChange: handleSetConstValue(props), strings: props.strings }));
    }
}
/**
 * Renders an indicator that signifies that the value
 * displayed is a default value
 */
export function renderDefaultValueIndicator(props) {
    if (typeof props.default !== "undefined" && !isEqual(props.data, props.default)) {
        return (React.createElement(DefaultValue, { className: props.className, disabled: props.disabled, onChange: handleSetDefaultValue(props), strings: props.strings }));
    }
}
export function handleChange(props) {
    return (config) => {
        props.onChange({
            dataLocation: props.dataLocation,
            dictionaryId: props.dictionaryId,
            value: config.value,
            isArray: config.isArray,
            isLinkedData: config.isLinkedData,
            linkedDataAction: config.linkedDataAction,
            index: config.index,
        });
    };
}
export function handleSetDefaultValue(props) {
    return function () {
        props.onChange({
            dataLocation: props.dataLocation,
            dictionaryId: props.dictionaryId,
            value: props.default,
        });
    };
}
export function handleSetConstValue(props) {
    return function () {
        props.onChange({
            dataLocation: props.dataLocation,
            dictionaryId: props.dictionaryId,
            value: props.const,
        });
    };
}
function handleRemove(props) {
    return function () {
        return props.onChange({
            dataLocation: props.dataLocation,
            dictionaryId: props.dictionaryId,
            value: undefined,
        });
    };
}
/**
 * Renders an invalid message
 */
export function renderInvalidMessage(props) {
    if (props.invalidMessage !== "" && props.displayValidationInline) {
        return React.createElement("div", { className: props.className }, props.invalidMessage);
    }
}
/**
 * Renders an indicator that a field is required
 */
export function renderRequired(props) {
    return props.required ? React.createElement("span", { className: props.className }, "(required)") : null;
}
/**
 * updates the validity
 *
 * Use:
 * useEffect(() => {
 *    updateValidity();
 * });
 */
export function updateValidity(props) {
    return function (refs) {
        const formControlElement = props.ref.current;
        if (formControlElement !== null && typeof formControlElement !== "undefined") {
            props.ref.current.setCustomValidity(props.invalidMessage);
        }
        if (Array.isArray(refs)) {
            refs.forEach(ref => {
                ref.current.setCustomValidity(props.invalidMessage);
            });
        }
    };
}
/**
 * Reports the current validity of the form item
 */
function reportValidity(props) {
    return function () {
        updateValidity(props);
        if (props.displayValidationBrowserDefault) {
            props.ref.current.reportValidity();
        }
    };
}
export function getConfig(props) {
    return {
        type: props.type,
        schemaDictionary: props.schemaDictionary,
        dataLocation: props.dataLocation,
        navigationConfigId: props.navigationConfigId,
        dictionaryId: props.dictionaryId,
        dataDictionary: props.dataDictionary,
        navigation: props.navigation,
        schemaLocation: props.schemaLocation,
        disabled: props.disabled,
        value: props.data,
        schema: props.schema,
        elementRef: props.ref,
        reportValidity: reportValidity(props),
        updateValidity: updateValidity(props),
        onChange: props.type === ControlType.section ? props.onChange : handleChange(props),
        min: props.min,
        max: props.max,
        step: props.step,
        rows: props.rows,
        options: props.options,
        label: props.label,
        onUpdateSection: props.onUpdateSection,
        invalidMessage: props.invalidMessage,
        validationErrors: props.validationErrors,
        displayValidationBrowserDefault: props.displayValidationBrowserDefault,
        displayValidationInline: props.displayValidationInline,
        displayValidationErrorList: props.displayValidationErrorList,
        minItems: props.minItems,
        maxItems: props.maxItems,
        onAddExampleData: props.onAddExampleData,
        default: props.default,
        component: props.component,
        required: props.required,
        controls: props.controls,
        controlComponents: props.controlComponents,
        controlPlugins: props.controlPlugins,
        untitled: props.untitled,
        messageSystem: props.messageSystem,
        strings: props.strings,
        messageSystemOptions: props.messageSystemOptions,
        categories: props.categories,
        validate: props.validate,
    };
}
/**
 * Explicitly updates the default value as the value
 */
export function handleUpdateValueToDefaultValue(props) {
    return function () {
        props.onChange({
            dataLocation: props.dataLocation,
            dictionaryId: props.dictionaryId,
            value: props.default,
        });
    };
}
//# sourceMappingURL=template.control.utilities.js.map