import React, { useEffect, useRef, useState } from "react";
import { canUseDOM } from "exenv-es6";
import rafThrottle from "raf-throttle";
import { ResizeHandleLocation, ViewerCustomAction, } from "./viewer.props";
import { MessageSystemType } from "design-to-code";
export function Viewer(props) {
    let iframeRef = useRef(null);
    const [resizing, setResizing] = useState(false);
    const [dragReferenceY, setDragReferenceY] = useState(null);
    const [dragReferenceX, setDragReferenceX] = useState(null);
    const [dragHandleLocation, setDragHandleLocation] = useState(null);
    const messageSystemConfig = {
        onMessage: handleMessageSystem,
    };
    const throttledHandleMouseMove = rafThrottle(handleMouseMove);
    useEffect(() => {
        document.addEventListener("mouseup", handleMouseUp);
        document.addEventListener("mousemove", throttledHandleMouseMove);
        iframeRef.current.contentWindow.addEventListener("message", handleIframeMessage);
        if (props.messageSystem !== undefined) {
            props.messageSystem.add(messageSystemConfig);
        }
        return function cleanup() {
            document.removeEventListener("mouseup", handleMouseUp);
            document.removeEventListener("mousemove", throttledHandleMouseMove);
            iframeRef.current.contentWindow.removeEventListener("message", handleIframeMessage);
            if (props.messageSystem !== undefined) {
                props.messageSystem.remove(messageSystemConfig);
            }
        };
    });
    function renderResponsiveLeftHandle() {
        if (props.responsive) {
            return (React.createElement("button", { className: `dtc-viewer-handle dtc-viewer-handle__left`, onMouseDown: handleMouseDown(ResizeHandleLocation.left) }));
        }
    }
    function renderResponsiveRightHandle() {
        if (props.responsive) {
            return (React.createElement("button", { className: `dtc-viewer-handle dtc-viewer-handle__right`, onMouseDown: handleMouseDown(ResizeHandleLocation.right) }));
        }
    }
    function renderResponsiveBottomRow() {
        if (props.responsive) {
            return (React.createElement(React.Fragment, null,
                React.createElement("button", { className: `dtc-viewer-handle dtc-viewer-handle__bottom-left`, onMouseDown: handleMouseDown(ResizeHandleLocation.bottomLeft) }),
                React.createElement("button", { className: `dtc-viewer-handle dtc-viewer-handle__bottom`, "aria-hidden": true, onMouseDown: handleMouseDown(ResizeHandleLocation.bottom) }),
                React.createElement("button", { className: `dtc-viewer-handle dtc-viewer-handle__bottom-right`, onMouseDown: handleMouseDown(ResizeHandleLocation.bottomRight) })));
        }
    }
    function getHeight() {
        if (props.height) {
            return {
                height: `${props.height}px`,
            };
        }
    }
    function getWidth() {
        if (props.width) {
            return {
                width: `${props.width}px`,
            };
        }
    }
    function generateContentRegionClassNames() {
        let classes = "dtc-viewer_content-region";
        if (props.preview) {
            classes += ` ${"dtc-viewer_content-region__preview"}`;
        }
        if (resizing) {
            classes += ` ${"dtc-viewer_content-region__disabled"}`;
        }
        return classes;
    }
    function postMessage(message) {
        var _a;
        if (canUseDOM() && ((_a = iframeRef === null || iframeRef === void 0 ? void 0 : iframeRef.current) === null || _a === void 0 ? void 0 : _a.contentWindow)) {
            iframeRef.current.contentWindow.postMessage(JSON.stringify(message), "*");
        }
    }
    function handleIframeMessage(e) {
        if (e.data &&
            e.data.type === MessageSystemType.custom &&
            e.data.action === ViewerCustomAction.call) {
            e.stopImmediatePropagation();
            props.messageSystem.postMessage(Object.assign(Object.assign({}, e.data), { action: ViewerCustomAction.response }));
        }
    }
    function handleMessageSystem(e) {
        postMessage(e.data);
    }
    /**
     * Handle mouseUp
     */
    function handleMouseUp(e) {
        // only listen for left click
        if (e.button !== 0) {
            return;
        }
        setResizing(false);
        setDragReferenceY(null);
        setDragReferenceX(null);
        setDragHandleLocation(null);
    }
    function handleMouseMove(e) {
        if (!resizing) {
            return;
        }
        const heightOffset = dragReferenceY - e.pageY;
        const widthOffset = (dragReferenceX - e.pageX) * 2;
        const updatedHeight = props.height - heightOffset;
        const updatedWidthLeft = props.width + widthOffset;
        const updatedWidthRight = props.width - widthOffset;
        switch (dragHandleLocation) {
            case ResizeHandleLocation.bottom:
                setDragReferenceY(e.pageY);
                handleUpdateHeight(updatedHeight);
                break;
            case ResizeHandleLocation.bottomLeft:
                setDragReferenceY(e.pageY);
                setDragReferenceX(e.pageX);
                handleUpdateHeight(updatedHeight);
                handleUpdateWidth(updatedWidthLeft);
                break;
            case ResizeHandleLocation.bottomRight:
                setDragReferenceY(e.pageY);
                setDragReferenceX(e.pageX);
                handleUpdateHeight(updatedHeight);
                handleUpdateWidth(updatedWidthRight);
                break;
            case ResizeHandleLocation.left:
                setDragReferenceX(e.pageX);
                handleUpdateWidth(updatedWidthLeft);
                break;
            case ResizeHandleLocation.right:
                setDragReferenceX(e.pageX);
                handleUpdateWidth(updatedWidthRight);
                break;
        }
    }
    function handleMouseDown(handleLocation) {
        return (e) => {
            // only listen for left click
            if (e.button !== 0) {
                return;
            }
            switch (handleLocation) {
                case ResizeHandleLocation.bottom:
                    handleBottomMouseDown(e);
                    break;
                case ResizeHandleLocation.bottomRight:
                    handleBottomRightMouseDown(e);
                    break;
                case ResizeHandleLocation.bottomLeft:
                    handleBottomLeftMouseDown(e);
                    break;
                case ResizeHandleLocation.left:
                    handleLeftMouseDown(e);
                    break;
                case ResizeHandleLocation.right:
                    handleRightMouseDown(e);
                    break;
            }
        };
    }
    function handleBottomMouseDown(e) {
        setResizing(true);
        setDragReferenceY(e.pageY);
        setDragHandleLocation(ResizeHandleLocation.bottom);
    }
    function handleBottomRightMouseDown(e) {
        setResizing(true);
        setDragReferenceY(e.pageY);
        setDragReferenceX(e.pageX);
        setDragHandleLocation(ResizeHandleLocation.bottomRight);
    }
    function handleBottomLeftMouseDown(e) {
        setResizing(true);
        setDragReferenceY(e.pageY);
        setDragReferenceX(e.pageX);
        setDragHandleLocation(ResizeHandleLocation.bottomLeft);
    }
    function handleLeftMouseDown(e) {
        setResizing(true);
        setDragReferenceX(e.pageX);
        setDragHandleLocation(ResizeHandleLocation.left);
    }
    function handleRightMouseDown(e) {
        setResizing(true);
        setDragReferenceX(e.pageX);
        setDragHandleLocation(ResizeHandleLocation.right);
    }
    function handleUpdateHeight(height) {
        if (props.onUpdateHeight) {
            props.onUpdateHeight(height);
        }
    }
    function handleUpdateWidth(width) {
        if (props.onUpdateWidth) {
            props.onUpdateWidth(width);
        }
    }
    return (React.createElement("div", { className: "dtc-viewer" },
        React.createElement("div", { className: generateContentRegionClassNames(), style: Object.assign(Object.assign({}, getHeight()), getWidth()) },
            renderResponsiveLeftHandle(),
            React.createElement("base", { target: "_blank" }),
            React.createElement("iframe", { ref: iframeRef, className: "dtc-viewer_iframe", src: props.iframeSrc }, "Your browser does not support iframes."),
            renderResponsiveRightHandle(),
            renderResponsiveBottomRow())));
}
//# sourceMappingURL=viewer.base.js.map